//
// Adventure Game for Cybiko by ssjx (2015)
//

#include <cybiko.h>
#include "bmp.h"

#define MAPSIZE 32

struct bitmap_info bminfo_mine;
//
void spike_add(int,int);

// utils.c
long swp(long num);

// mine.c (here!)
int mine_get(int,int);
void mine_set(int,int,int);

//
char mine[1024];
char mine_bk[1024];
char mine_dif[1024]; // changed areas? redraw just this?

//

struct Bitmap bmp_wall,bmp_rock,bmp_earth,bmp_diamond,bmp_exit,bmp_clear;

//
int changed=0;

//
// Load our graphics
//

void mine_gfx()
{
	struct Input * ptr_input;

	//	
	Bitmap_ctor(&bmp_earth);
	
	ptr_input = open_resource_Ex( "grass.pic" );
	if (!Bitmap_load( &bmp_earth, ptr_input ))
	{
	TRACE("Error loading");
	}
	Input_dtor( ptr_input, FREE_MEMORY );
	
	//	
	Bitmap_ctor(&bmp_wall);
	
	ptr_input = open_resource_Ex( "wall.pic" );
	if (!Bitmap_load( &bmp_wall, ptr_input ))
	{
	TRACE("Error loading");
	}
	Input_dtor( ptr_input, FREE_MEMORY );
	
	//	
	Bitmap_ctor(&bmp_rock);
	
	ptr_input = open_resource_Ex( "rock.pic" );
	if (!Bitmap_load( &bmp_rock, ptr_input ))
	{
	TRACE("Error loading");
	}
	Input_dtor( ptr_input, FREE_MEMORY );
	
	//	
	Bitmap_ctor(&bmp_diamond);
	
	ptr_input = open_resource_Ex( "diamond.pic" );
	if (!Bitmap_load( &bmp_diamond, ptr_input ))
	{
	TRACE("Error loading");
	}
	Input_dtor( ptr_input, FREE_MEMORY );
	
	//	
	Bitmap_ctor(&bmp_exit);
	
	ptr_input = open_resource_Ex( "exit.pic" );
	if (!Bitmap_load( &bmp_exit, ptr_input ))
	{
	TRACE("Error loading");
	}
	Input_dtor( ptr_input, FREE_MEMORY );
	
		//	
	Bitmap_ctor(&bmp_clear);
	
	ptr_input = open_resource_Ex( "path.pic" );
	if (!Bitmap_load( &bmp_clear, ptr_input ))
	{
	TRACE("Error loading path");
	}
	Input_dtor( ptr_input, FREE_MEMORY );
	

	
}

void mine_save()
{
memcpy(mine_bk,mine,sizeof(mine_bk));
}

void mine_restore()
{
memcpy(mine,mine_bk,sizeof(mine));
}

void mine_load()
{
	int i,j,c,pos;
	long offset;	
	char *tmp;
	
	struct bitmap_header bmheader;
	
	struct Input *bmp_file_input;
	
	tmp=(char *)malloc(MAPSIZE*MAPSIZE); 
	
	if (tmp==0)
	{
		TRACE("Not Enough Memory (TMP Buffer)..");
		return;
	}
	
	TRACE("Loading MINE map..");
	
	///
	// Load map from a resource
	//
	// Can't seek within a bmp so am just skipping until i get to data
	//
	bmp_file_input=open_resource_Ex( "mine.bmp" );
	Input_read( bmp_file_input, tmp,78);
	Input_read( bmp_file_input, tmp,1000);
	Input_read( bmp_file_input, tmp,MAPSIZE*MAPSIZE); // <<-- Actual map!
	Input_dtor( bmp_file_input, FREE_MEMORY );
	
	c=0;
	pos=0;
	for(j=0;j<MAPSIZE;j++)
	{
		for(i=0;i<MAPSIZE;i++)
		{
		pos=(MAPSIZE-1-j)*MAPSIZE;
		mine[pos+i]=tmp[c];
		c++;
		}
	}
	
	free(tmp);
	
	//
	// Sort out attributes
	//
	c=0;
	for(j=0;j<MAPSIZE;j++)
	{
		for(i=0;i<MAPSIZE;i++)
		{
			// Add spike!
			if (mine[c]==5)
			{
				spike_add(i,j);
				mine[c]=0;
				
			}
			c++;
		}
	}
}

void mine_redraw(struct module_t *ptr_main_module)
{
  DisplayGraphics_page_copy( ptr_main_module->m_gfx, 1, 0, 0, 0, 160, 100 );
}

void mine_draw(struct module_t *ptr_main_module,int x,int y)
{
	int i,j,pos,spos;
	int spr,c;
 	
     	DisplayGraphics_set_page( ptr_main_module->m_gfx, 1 );
     	
     	TGraph_set_color( ptr_main_module->m_gfx, CLR_WHITE );
	TGraph_fill_rect(ptr_main_module->m_gfx, 0, 0, 160, 100 );
     	
     	spos=(MAPSIZE*(y*6))+(x*10);
     	
	for(j=0;j<6;j++)
	{
		pos=spos+(j<<5);//	Mapsize=32
		
		for(i=0;i<10;i++)
		{
			spr=mine[pos];
			mine_dif[pos]=0;
			// <<4 = *16
			switch(spr)
			{
				case 0:
					// Empty
					 DisplayGraphics_draw_bitmap( ptr_main_module->m_gfx, &bmp_clear, i<<4,j<<4, BM_NORMAL );
				break;
				
				case 1:
					// Earth
					 DisplayGraphics_draw_bitmap( ptr_main_module->m_gfx, &bmp_earth, i<<4,j<<4, BM_NORMAL );
				break;
				
				case 2:
					// Wall 
					 DisplayGraphics_draw_bitmap( ptr_main_module->m_gfx, &bmp_wall, i<<4,j<<4, BM_NORMAL );
				break;
				
				case 3:
					// Diamond 
					 DisplayGraphics_draw_bitmap( ptr_main_module->m_gfx, &bmp_diamond, i<<4,j<<4, BM_NORMAL );
				break;
				
				case 4:
					// Rock
					 DisplayGraphics_draw_bitmap( ptr_main_module->m_gfx, &bmp_rock, i<<4,j<<4, BM_NORMAL );
				break;
				
				case 5:
					// Spike
				break;
				
				case 6:
					// Exit
					 DisplayGraphics_draw_bitmap( ptr_main_module->m_gfx, &bmp_exit, i<<4,j<<4, BM_NORMAL );
				break;
			}
			pos++;
		}
	}
	
	
     	DisplayGraphics_set_page( ptr_main_module->m_gfx, 0 );
}


void mine_smartdraw(struct module_t *ptr_main_module,int x,int y)
{
	int i,j,pos,spos;
	int spr,c;
	int dif;
	
 	if (changed==0){return;}
 	changed=0;
 		
     	DisplayGraphics_set_page( ptr_main_module->m_gfx, 1 );
     	
     	spos=(MAPSIZE*(y*6))+(x*10);
     	
	for(j=0;j<6;j++)
	{
		pos=spos+(j<<5);//	Mapsize=32
		
		for(i=0;i<10;i++)
		{
		
			if (mine_dif[pos]==1)
			{
				mine_dif[pos]=0;
			
				spr=mine[pos];
				// <<4 = *16
				switch(spr)
				{
					case -1:
					case 0:
						// Empty
						 DisplayGraphics_draw_bitmap( ptr_main_module->m_gfx, &bmp_clear, i<<4,j<<4, BM_NORMAL );
					break;
					
					case 1:
						// Earth
						 DisplayGraphics_draw_bitmap( ptr_main_module->m_gfx, &bmp_earth, i<<4,j<<4, BM_NORMAL );
					break;
					
					case 2:
						// Wall 
						 DisplayGraphics_draw_bitmap( ptr_main_module->m_gfx, &bmp_wall, i<<4,j<<4, BM_NORMAL );
					break;
					
					case 3:
						// Diamond 
						 DisplayGraphics_draw_bitmap( ptr_main_module->m_gfx, &bmp_diamond, i<<4,j<<4, BM_NORMAL );
					break;
					
					case 4:
						// Rock
						 DisplayGraphics_draw_bitmap( ptr_main_module->m_gfx, &bmp_rock, i<<4,j<<4, BM_NORMAL );
					break;
					
					case 5:
						// Spike
					break;
					
					case 6:
						// Exit
						 DisplayGraphics_draw_bitmap( ptr_main_module->m_gfx, &bmp_exit, i<<4,j<<4, BM_NORMAL );
					break;
					
				}
			}
			pos++;
		}
	}
	
     	DisplayGraphics_set_page( ptr_main_module->m_gfx, 0 );
}



int mine_update()
{
	int i,j;
	int again=0;
	
	do
	{
		again=0;
		
		for(j=0;j<MAPSIZE;j++)
		{	
			for(i=0;i<MAPSIZE;i++)
			{
				if (mine_get(i,j)==4)
				{
					// Space below rock
					if (mine_get(i,j+1)==0)
					{
						mine_set(i,j,0);
						mine_set(i,j+1,4);
						
						again=1;
					}
					
					// On diamond, is space clear to side?
					if (mine_get(i,j+1)==3)
					{
						// Right clear?
						if (mine_get(i+1,j)==0 && mine_get(i+1,j+1)<=0)
						{
							mine_set(i,j,0);
							
							if (mine_get(i+1,j+1)==-1)
							{
								mine_set(i+1,j+1,4);
								return -1;
							}
							
							mine_set(i+1,j+1,4);	
							again=1;
						}
						else
						{
							// Left clear?
							if (mine_get(i-1,j)==0 && mine_get(i-1,j+1)<=0)
							{
								mine_set(i,j,0);
								
								if (mine_get(i-1,j+1)==-1)
								{
									mine_set(i-1,j+1,4);
									return -1;
								}
								
								mine_set(i-1,j+1,4);
								again=1;	
							}
						}
					}		
				}
			}
		}
		
	// Loop in case we have a chain of falling rocks...
	}while(again==1);

return 0;
}

int mine_get(int x,int y)
{
	int pos=(y<<5)+x;	//32 == <<5
	return mine[pos];
}

void mine_set(int x,int y,int c)
{
	int pos=(y<<5)+x;	//32 == <<5
		
	if (mine[pos]!=c)
	{
		mine[pos]=c;
		mine_dif[pos]=1;
		changed=1;
	}	
}




