//
// Adventure Game for Cybiko by ssjx (2015)
//
// ** Mountain **
//
//
// 256 BMP loader
//
#include <cybiko.h>
#include "bmp.h"

#define MAPSIZE 32

struct bitmap_info bminfo_mountain;
//
void bug_add(int,int);

// utils.c
long swp(long num);

// mine.c (here!)
int mountain_get(int,int);
void mountain_set(int,int,int);

//
char mountain[1024];
char mountain_bk[1024];
//

struct Bitmap bmp_block,bmp_ladder,bmp_pillar,bmp_prof,bmp_mboulder;

//
// Load our graphics
//

void mountain_gfx()
{
	struct Input * ptr_input;

	//	
	Bitmap_ctor(&bmp_block);	
	ptr_input = open_resource_Ex( "wall.pic" );
	if (!Bitmap_load( &bmp_block, ptr_input ))
	{
		TRACE("Error loading block(mountain)");
	}
	Input_dtor( ptr_input, FREE_MEMORY );
	//
	
	Bitmap_ctor(&bmp_ladder);	
	ptr_input = open_resource_Ex( "ladder.pic" );
	if (!Bitmap_load( &bmp_ladder, ptr_input ))
	{
		TRACE("Error loading ladder");
	}
	Input_dtor( ptr_input, FREE_MEMORY );
	//
	
	Bitmap_ctor(&bmp_pillar);	
	ptr_input = open_resource_Ex( "pillar.pic" );
	if (!Bitmap_load( &bmp_pillar, ptr_input ))
	{
		TRACE("Error loading pillar");
	}
	Input_dtor( ptr_input, FREE_MEMORY );
	
	//
	
	Bitmap_ctor(&bmp_mboulder);	
	ptr_input = open_resource_Ex( "mboulder.pic" );
	if (!Bitmap_load( &bmp_mboulder, ptr_input ))
	{
		TRACE("Error loading boulder");
	}
	Input_dtor( ptr_input, FREE_MEMORY );	

	//
	
	Bitmap_ctor(&bmp_prof);	
	ptr_input = open_resource_Ex( "prof.pic" );
	if (!Bitmap_load( &bmp_prof, ptr_input ))
	{
		TRACE("Error loading prof");
	}
	Input_dtor( ptr_input, FREE_MEMORY );

	
}

void mountain_save()
{
memcpy(mountain_bk,mountain,sizeof(mountain_bk));
}

void mountain_restore()
{
memcpy(mountain,mountain_bk,sizeof(mountain));
}


void mountain_load()
{
	int i,j,c,pos;
	long offset;	
	char *tmp;
	
	struct bitmap_header bmheader;

	struct Input *bmp_file_input;
	
	tmp=(char *)malloc(MAPSIZE*MAPSIZE); 
	
	if (tmp==0)
	{
		TRACE("Not Enough Memory (TMP Buffer)..");
		return;
	}
	
	TRACE("Loading MOUNTAIN map..");
	
	bmp_file_input=open_resource_Ex( "mountain.bmp" );
	Input_read( bmp_file_input, tmp,78);
	Input_read( bmp_file_input, tmp,1000);
	Input_read( bmp_file_input, tmp,MAPSIZE*MAPSIZE); // <<-- Actual map!
	Input_dtor( bmp_file_input, FREE_MEMORY );
	
	c=0;
	pos=0;
	for(j=0;j<MAPSIZE;j++)
	{
		for(i=0;i<MAPSIZE;i++)
		{
		pos=(MAPSIZE-1-j)*MAPSIZE;
		mountain[pos+i]=tmp[c];
		c++;
		}
	}
	
	free(tmp);
	
	//
	// Add bugs!
	//
	c=0;
	for(j=0;j<MAPSIZE;j++)
	{
		for(i=0;i<MAPSIZE;i++)
		{
			// Add bugs
			if (mountain[c]==5)
			{
				bug_add(i,j);
				mountain[c]=0;
				
			}
			c++;
		}
	}
}

void mountain_redraw(struct module_t *ptr_main_module)
{
	DisplayGraphics_page_copy( ptr_main_module->m_gfx, 1, 0, 0, 0, 160, 100 );
}

void mountain_draw(struct module_t *ptr_main_module,int x,int y)
{
	int i,j,pos=0,spos;
	int spr,c;
	
     	DisplayGraphics_set_page( ptr_main_module->m_gfx, 1 );
     	
     	TGraph_set_color( ptr_main_module->m_gfx, CLR_WHITE );
	TGraph_fill_rect(ptr_main_module->m_gfx, 0, 0, 160, 100 );
     	
     	spos=(MAPSIZE*(y*6))+(x*10);
     	
	for(j=0;j<6;j++)
	{
		pos=spos+(j<<5);//	Map width=32
		
		for(i=0;i<10;i++)
		{
			spr=mountain[pos];
			// <<4 = *16
			switch(spr)
			{
				case 0:
					// Empty
					//Graphics_draw_bitmap( ptr_main_module->m_gfx, &bmp_grass, i<<4,j<<4, BM_NORMAL );
				break;
				
				case 1:
					// wall
					 DisplayGraphics_draw_bitmap( ptr_main_module->m_gfx, &bmp_block, i<<4,j<<4, BM_NORMAL );
				break;
				
				case 2:
					// ladder 
					 DisplayGraphics_draw_bitmap( ptr_main_module->m_gfx, &bmp_ladder, i<<4,j<<4, BM_NORMAL );
				break;
				
				case 3:
					// pillar
					 DisplayGraphics_draw_bitmap( ptr_main_module->m_gfx, &bmp_pillar, i<<4,j<<4, BM_NORMAL );
				break;
				
				case 4:
					// prof
					 DisplayGraphics_draw_bitmap( ptr_main_module->m_gfx, &bmp_prof, i<<4,j<<4, BM_NORMAL );
				break;
				
				case 5:
					// Bug!
				break;
				
				case 6:
					// Bug!
					DisplayGraphics_draw_bitmap( ptr_main_module->m_gfx, &bmp_mboulder, i<<4,j<<4, BM_NORMAL );
				break;
			}
			pos++;
			
			if (pos>(MAPSIZE*MAPSIZE))
			{
			TRACE("Error! Mountain Draw Overflow!");
			}
		}
	}
	
     	DisplayGraphics_set_page( ptr_main_module->m_gfx, 0 );
}

void mountain_replace(int a,int b)
{
	int i,j,c=0;

	for(j=0;j<MAPSIZE;j++)
	{
		for(i=0;i<MAPSIZE;i++)
		{
			if (mountain[c]==a)
			{
			
				mountain[c]=b;
			}
		
			c++;
		}
	}
}

int mountain_get(int x,int y)
{
	int pos=(y*MAPSIZE)+x;	//32 == <<5
	return mountain[pos];
}

void mountain_set(int x,int y,int c)
{
	int pos=(y*MAPSIZE)+x;	//32 == <<5
	
	if (pos<(MAPSIZE*MAPSIZE))
	{
		mountain[pos]=c;
	}
	else
	{
		TRACE("Error! Mountain Set Overflow");
	}
	
}


